package edu.lawrence.androiddirectoryonline

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.animateContentSize
import androidx.compose.animation.core.tween
import androidx.compose.animation.fadeOut
import androidx.compose.animation.shrinkVertically
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.Settings
import androidx.compose.material3.Button
import androidx.compose.material3.DismissDirection
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SwipeToDismiss
import androidx.compose.material3.SwipeToDismissBox
import androidx.compose.material3.SwipeToDismissBoxState
import androidx.compose.material3.SwipeToDismissBoxValue
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.rememberSwipeToDismissBoxState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.key
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberUpdatedState
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import edu.lawrence.androiddirectoryonline.ui.theme.AndroidDirectoryOnlineTheme
import kotlinx.coroutines.delay

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        setContent {
            AndroidDirectoryOnlineTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    DirectoryApp(
                        modifier = Modifier.padding(innerPadding)
                    )
                }
            }
        }
    }
}

@Composable
fun DirectoryApp(modifier: Modifier = Modifier) {
    val vm : DirectoryModel = viewModel()
    val navController = rememberNavController()
    NavHost(navController,startDestination="login") {
        composable(route="login") {
            LoginView(vm,toMain = { navController.navigate("directory")}, modifier = modifier)
        }
        composable(route="directory") {
            DirectoryScreen(vm,toCreate = { navController.navigate("create")}, modifier = modifier)
        }
        composable(route="create") {
            CreateView(vm,toMain = { navController.navigate("directory")}, modifier = modifier)
        }
    }
}

@Composable
fun LoginView(vm : DirectoryModel,toMain : () -> Unit,modifier:Modifier) {
    val focusManager = LocalFocusManager.current
    val name = remember { mutableStateOf("") }
    val password = remember { mutableStateOf("") }
    Column(horizontalAlignment = Alignment.End, verticalArrangement = Arrangement.Center, modifier = modifier
        .padding(horizontal = 20.dp)
        .fillMaxSize()) {
        Row(horizontalArrangement = Arrangement.End, verticalAlignment = Alignment.Bottom) {
            Text("User")
            Spacer(modifier=Modifier.width(10.dp))
            TextField(value = name.value,
                onValueChange = {newValue : String -> name.value = newValue},
                keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                keyboardActions = KeyboardActions(
                    onDone = { focusManager.clearFocus() })
            )
        }
        Spacer(modifier=Modifier.height(10.dp))
        Row(horizontalArrangement = Arrangement.End, verticalAlignment = Alignment.Bottom) {
            Text("Password")
            Spacer(modifier=Modifier.width(10.dp))
            TextField(value = password.value,
                onValueChange = {newValue : String -> password.value = newValue},
                visualTransformation = PasswordVisualTransformation(),
                keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Password,imeAction = ImeAction.Done),
                keyboardActions = KeyboardActions(
                    onDone = { focusManager.clearFocus() })
            )
        }
        Spacer(modifier=Modifier.height(20.dp))
        Row() {
            Button(onClick = {vm.newUser(name.value,password.value,toMain)}) {
                Text("New User")
            }
            Spacer(modifier=Modifier.fillMaxSize(0.5F))
            Button(onClick = {vm.login(name.value,password.value,toMain)}) {
                Text("Log In")
            }

        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun DirectoryScreen(vm : DirectoryModel,toCreate : () -> Unit,modifier:Modifier) {
    Scaffold(topBar = {
        TopAppBar(
            title = {
                Text(
                    "Online Directory",
                )
            },
            actions = {
                IconButton(onClick = toCreate) {
                    Icon(
                        imageVector = Icons.Default.Add,
                        contentDescription = "Add new"
                    )
                }
            },
        )
    },
        modifier = Modifier.fillMaxSize())
    { innerPadding ->
        DirectoryView(vm,modifier.padding(innerPadding))
    }
}

@Composable
fun DirectoryView(vm : DirectoryModel,modifier:Modifier) {
    val dirList = remember { vm.people }
    if(dirList.value == null) {
        Text("Loading...")
    } else {
        val people = dirList.value!!
        Column(verticalArrangement = Arrangement.Top, modifier = modifier) {
            people.forEach { person ->
                key(person.id) {
                    PersonItem(person = person, onRemove = { vm.removePerson(it.id) })
                }
            }
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun PersonItem(
    person : Person,
    onRemove: (Person) -> Unit
) {
    val dismissState = rememberSwipeToDismissBoxState(
        confirmValueChange = {
            if(it == SwipeToDismissBoxValue.EndToStart) {
                onRemove(person)
                return@rememberSwipeToDismissBoxState true
            }
            return@rememberSwipeToDismissBoxState false
        },
        positionalThreshold = { it * .33f }
    )
    SwipeToDismissBox(
        state = dismissState,
        modifier = Modifier.fillMaxWidth().wrapContentHeight(),
        backgroundContent = { DismissBackground(dismissState) },
        content = {
            Row(modifier = Modifier
                .fillMaxWidth()
                .background(MaterialTheme.colorScheme.background)
                .padding(horizontal = 16.dp,vertical=4.dp)) {
                Text(person.name)
                Spacer(modifier=Modifier.width(20.dp))
                Text(person.office)
            }
        }
    )
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun DismissBackground(dismissState: SwipeToDismissBoxState) {
    val color = if (dismissState.dismissDirection == SwipeToDismissBoxValue.EndToStart) {
        Color.Red
    } else Color.Transparent

    Box(
        modifier = Modifier
            .fillMaxWidth()
            .background(color)
            .padding(top=4.dp,bottom=4.dp,end=6.dp),
        contentAlignment = Alignment.CenterEnd
    ) {
        Icon(
            imageVector = Icons.Default.Delete,
            contentDescription = null,
            tint = Color.White
        )
    }
}

@Composable
fun CreateView(vm : DirectoryModel,toMain : () -> Unit,modifier:Modifier) {
    val focusManager = LocalFocusManager.current
    val name = remember { mutableStateOf("") }
    val office = remember { mutableStateOf("") }
    Column(horizontalAlignment = Alignment.End, verticalArrangement = Arrangement.Center, modifier = modifier
        .padding(horizontal = 20.dp)
        .fillMaxSize()) {
        Row(horizontalArrangement = Arrangement.End, verticalAlignment = Alignment.Bottom) {
            Text("Name")
            Spacer(modifier=Modifier.width(10.dp))
            TextField(value = name.value,
                onValueChange = {newValue : String -> name.value = newValue},
                keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                keyboardActions = KeyboardActions(
                    onDone = { focusManager.clearFocus() })
            )
        }
        Row(horizontalArrangement = Arrangement.End, verticalAlignment = Alignment.Bottom) {
            Text("Office")
            Spacer(modifier=Modifier.width(10.dp))
            TextField(value = office.value,
                onValueChange = {newValue : String -> office.value = newValue},
                keyboardOptions = KeyboardOptions(imeAction = ImeAction.Done),
                keyboardActions = KeyboardActions(
                    onDone = { focusManager.clearFocus() })
            )
        }
        Spacer(modifier=Modifier.height(20.dp))
        Row() {
            Button(onClick = { toMain() }) {
                Text("Cancel")
            }
            Spacer(modifier=Modifier.fillMaxSize(0.5F))
            Button(onClick = {vm.addPerson(Person(name.value,office.value),toMain)}) {
                Text("Add Person")
            }
        }
    }
}
